@@ -491,9 +491,9 @@ void addUnorderedOps(venv &ve, ty *t1, ty *t2, ty *t3, ty *t4)
           formal(t3, SYM(b)));
   
   addCast(ve,t1,primFile(),read<T>);
-  addCast(ve,t2,primFile(),readArray<T>);
-  addCast(ve,t3,primFile(),readArray<T>);
-  addCast(ve,t4,primFile(),readArray<T>);
+  addCast(ve,t2,primFile(),readArray1<T>);
+  addCast(ve,t3,primFile(),readArray2<T>);
+  addCast(ve,t4,primFile(),readArray3<T>);
   
   addWrite(ve,write<T>,t1,t2);
   addRestFunc(ve,writeArray<T>,primVoid(),SYM(write),t3,
@@ -869,15 +869,15 @@ void base_venv(venv &ve)
   addConstant<Int>(ve, Int_MIN, primInt(), SYM(intMin));
   addConstant<double>(ve, HUGE_VAL, primReal(), SYM(inf));
   addConstant<double>(ve, run::infinity, primReal(), SYM(infinity));
+  addConstant<double>(ve, nan(""), primReal(), SYM(nan));
   addConstant<double>(ve, DBL_MAX, primReal(), SYM(realMax));
   addConstant<double>(ve, DBL_MIN, primReal(), SYM(realMin));
   addConstant<double>(ve, DBL_EPSILON, primReal(), SYM(realEpsilon));
   addConstant<Int>(ve, DBL_DIG, primInt(), SYM(realDigits));
-  addConstant<Int>(ve, RAND_MAX, primInt(), SYM(randMax));
+  addConstant<Int>(ve, RANDOM_MAX, primInt(), SYM(randMax));
   addConstant<double>(ve, PI, primReal(), SYM(pi));
-  addConstant<string>(ve, string(settings::VERSION)+string(SVN_REVISION),
-                      primString(),SYM(VERSION));
-  
+  addConstant<string>(ve, string(REVISION),primString(),SYM(VERSION));
+
   addVariable<pen>(ve, &processData().currentpen, primPen(), SYM(currentpen));
 
 #ifdef OPENFUNCEXAMPLE
