@@ -24,6 +24,8 @@ namespace camp {
 
 enum Interaction {EMBEDDED=0,BILLBOARD};
 
+void copyArray4x4C(double*& dest, const vm::array *a);
+  
 class box {
   pair p[4];
 public:
@@ -109,11 +111,33 @@ public:
   // Adjust the bbox of the picture based on the addition of this
   // element. The iopipestream is needed for determining label sizes.
   virtual void bounds(bbox&, iopipestream&, boxvector&, bboxlist&) {}
-  virtual void bounds(bbox3&) {}
+  virtual void bounds(const double*, bbox3&) {}
+  virtual void bounds(bbox3& b) { bounds(NULL, b); }
 
   // Compute bounds on ratio (x,y)/z for 3d picture (not cached).
+  virtual void ratio(const double *t, pair &b, double (*m)(double, double),
+                     double fuzz, bool &first) {}
+  
+  virtual void minratio(const double *t, pair &b, double fuzz, bool &first) {
+    ratio(t, b, camp::min, fuzz, first);
+  }
+  
+  virtual void maxratio(const double *t,pair &b, double fuzz, bool &first) {
+    ratio(t, b, camp::max, fuzz, first);
+  }
+  
   virtual void ratio(pair &b, double (*m)(double, double), double fuzz,
-                     bool &first){}
+                     bool &first) {
+    ratio(NULL, b, m, fuzz, first);
+  }
+
+  virtual void minratio(pair &b, double fuzz, bool &first) {
+    minratio(NULL, b, fuzz, first);
+  }
+
+  virtual void maxratio(pair &b, double fuzz, bool &first) {
+    maxratio(NULL, b, fuzz, first);
+  }
 
   virtual bool islabel() {return false;}
 
@@ -131,8 +155,12 @@ public:
   virtual bool endclip() {return false;}
   
   virtual bool begingroup() {return false;}
+  virtual bool begingroup3() {return false;}
 
   virtual bool endgroup() {return false;}
+  virtual bool endgroup3() {return false;}
+
+  virtual const double* transf3() {return NULL;}
 
   virtual void save(bool b) {}
   
@@ -147,29 +175,58 @@ public:
   }
 
   // Output to a PRC file
-  // The array origin contains the points about which to rotate billboard labels
-  virtual bool write(prcfile *out, unsigned int *count, vm::array *index,
-                     vm::array *origin, double compressionlimit,
+  virtual bool write(prcfile *out, unsigned int *count, double compressionlimit,
                      groupsmap& groups) {
     return false;
   }
-
+  
   // Used to compute deviation of a surface from a quadrilateral.
   virtual void displacement() {}
 
   // Render with OpenGL
   virtual void render(GLUnurbs *nurb, double size2, 
                       const triple& Min, const triple& Max,
-                      double perspective, bool transparent) {}
+                      double perspective, bool lighton, bool transparent) {}
 
   // Transform as part of a picture.
   virtual drawElement *transformed(const transform&) {
     return this;
   }
   
-  virtual drawElement *transformed(const vm::array&) {
+  virtual drawElement *transformed(const double* t) {
     return this;
   }
+
+};
+
+// Hold transform of an object.
+class drawElementLC : public virtual drawElement {
+public:
+  double *T; // Keep track of accumulative picture transform
+  
+  drawElementLC() : T(NULL) {}
+  
+  drawElementLC(const double *t) : T(NULL) {
+    copyTransform3(T,t);
+  }
+
+  drawElementLC(const vm::array& t) : T(NULL) {
+    copyArray4x4C(T,&t);
+  }
+
+  drawElementLC(const double* t, const drawElementLC *s) : T(NULL) {
+    multiplyTransform3(T,t,s->T);
+  }
+
+  virtual ~drawElementLC() {}
+
+  virtual bool is3D() {return true;}
+
+  virtual const double* transf3() {return T;}
+
+  virtual drawElement* transformed(const double* t) {
+    return new drawElementLC(t,this);
+  }
 };
 
 // Base class for drawElements that involve paths.
@@ -202,8 +259,7 @@ public:
   virtual void writeshiftedpath(texfile *out) {
     out->writeshifted(p);
   }
-  
-};
+};     
 
 // Base class for drawElements that involve paths and pens.
 class drawPathPenBase : public drawPathBase {
@@ -252,8 +308,6 @@ public:
     if (!pentype.getTransform().isIdentity())
       out->grestore();
   }
-  
-  
 };
   
 // Base class for drawElements that involve superpaths and pens.
@@ -330,4 +384,3 @@ GC_DECLARE_PTRFREE(camp::box);
 GC_DECLARE_PTRFREE(camp::drawElement);
 
 #endif
-
