@@ -22,7 +22,7 @@ public:
     return true;
   }
   
-  bool write(texfile *out) {
+  bool write(texfile *out, const bbox&) {
     out->grestore();
     return true;
   }
@@ -30,4 +30,6 @@ public:
 
 }
 
+GC_DECLARE_PTRFREE(camp::drawGrestore);
+
 #endif
