@@ -17,7 +17,6 @@ using namespace settings;
 namespace camp {
   
 string texready=string("(Please type a command or say `\\end')\n*");
-pen drawElement::lastpen;
 
 void drawLabel::labelwarning(const char *action) 
 {
@@ -25,95 +24,66 @@ void drawLabel::labelwarning(const char *action)
        << "\" " << action << " to avoid overwriting" << endl;
 }
  
-int drawLabel::wait(iopipestream &tex, const char *s, const char **abort,
-		    bool ignore)
+// Reads one of the dimensions from the pipe.
+void texdim(iopipestream& tex, double& dest, const string command,
+            const string name)
 {
-  int rc=tex.wait(s,abort);
-  if(rc > 0) {
-    if(fataltex[rc-1]) {
-      tex.pipeclose();
-      ignore=false;
-    } else {
-      tex << "\n";
-      tex.wait(s,abort);
-      tex << "\n";
-      tex.wait(s,abort);
-    }
-    if(!ignore) {
-      string s=tex.message();
-      tex.shred();
-      reportError(s);
-    }
-   }
-  tex.shred();
-  return rc;
-}
- 
-bool drawLabel::texbounds(iopipestream& tex, string& s, const char **abort,
-			  bool warn)
-{
-  string texbuf;
-  tex << "\\setbox\\ASYbox=\\hbox{" << stripblanklines(s) << "}\n\n";
-  int rc=wait(tex,texready.c_str(),abort,getSetting<bool>("inlinetex"));
-   if(rc) {
-     tex << "\\show 0\n";
-     tex.wait("\n*");
-     if(warn) {
-       if(getSetting<bool>("debug")) {
-	 ostringstream buf;
-	 buf << "Cannot determine size of label \"" << s << "\"";
-	 reportWarning(buf);
-       }
-       return false;
-     }
-     return false;
-   }
+  string start(">dim(");
+  string stop(")dim");
+  string expect("pt"+stop+"\n\n*");
 
-  tex << "\\showthe\\wd\\ASYbox\n";
-  tex >> texbuf;
-  
-  string cannotread="Cannot read label ";
-  if(texbuf[0] == '>' && texbuf[1] == ' ')
+  // ask the tex engine for dimension
+  tex << "\\immediate\\write16{" << start << "\\the\\" << command << "\\ASYbox"
+      << stop << "}\n";
+  // keep reading output until ')dim\n\n*' is read
+  tex.wait(expect.c_str());
+  string buffer = tex.getbuffer();
+
+  size_t dim1=buffer.find(start);
+  size_t dim2=buffer.find("pt" + stop);
+  string cannotread="Cannot read label "+name;
+  if (dim1 != string::npos && dim2 != string::npos) {
+    string n=buffer.substr(dim1+start.size(),dim2-dim1-start.size());
     try {
-      width=lexical::cast<double>(texbuf.c_str()+2,true)*tex2ps;
+      dest=lexical::cast<double>(n,true)*camp::tex2ps;
     } catch(lexical::bad_cast&) {
-      reportError(cannotread+"width");
+      camp::reportError(cannotread);
     }
-  else reportError(cannotread+"width");
-  tex << "\n";
-  wait(tex,"\n*",abort);
-  
-  tex << "\\showthe\\ht\\ASYbox\n";
-  tex >> texbuf;
-  if(texbuf[0] == '>' && texbuf[1] == ' ')
-    try {
-    height=lexical::cast<double>(texbuf.c_str()+2,true)*tex2ps;
-    } catch(lexical::bad_cast&) {
-      reportError(cannotread+"height");
-    }
-  else reportError(cannotread+"height");
-  tex << "\n";
-  wait(tex,"\n*",abort);
-  
-  tex << "\\showthe\\dp\\ASYbox\n";
-  tex >> texbuf;
-  if(texbuf[0] == '>' && texbuf[1] == ' ')
-    try {
-    depth=lexical::cast<double>(texbuf.c_str()+2,true)*tex2ps;
-    } catch(lexical::bad_cast&) {
-      reportError(cannotread+"depth");
-    }
-  else reportError(cannotread+"depth");
-  tex << "\n";
-  wait(tex,"\n*",abort);
-     
-  return true;
+  } else {
+    camp::reportError(cannotread);
+  }
+}
+
+void texbounds(double& width, double& height, double& depth,
+               iopipestream& tex, string& s)
+{
+  tex << "\\setbox\\ASYbox=\\hbox{" << stripblanklines(s) << "}\n\n";
+  tex.wait(texready.c_str());
+  texdim(tex,width,"wd","width");
+  texdim(tex,height,"ht","height");
+  texdim(tex,depth,"dp","depth");
 }   
 
 inline double urand()
-{			  
-  static const double factor=2.0/RAND_MAX;
-  return rand()*factor-1.0;
+{                         
+  static const double factor=2.0/RANDOM_MAX;
+  return random()*factor-1.0;
+}
+
+void setpen(iopipestream& tex, const string& texengine, const pen& pentype) 
+{
+  bool Latex=latex(texengine);
+  
+  if(Latex && setlatexfont(tex,pentype,drawElement::lastpen)) {
+    tex << "\n";
+    tex.wait(texready.c_str());
+  }
+  if(settexfont(tex,pentype,drawElement::lastpen,Latex)) {
+    tex << "\n";
+    tex.wait(texready.c_str());
+  }
+  
+  drawElement::lastpen=pentype;
 }
 
 void drawLabel::getbounds(iopipestream& tex, const string& texengine)
@@ -121,43 +91,30 @@ void drawLabel::getbounds(iopipestream& tex, const string& texengine)
   if(havebounds) return;
   havebounds=true;
   
-  const char **abort=texabort(texengine);
-  if(pentype.size() != lastpen.size() ||
-     pentype.Lineskip() != lastpen.Lineskip()) {
-    if(latex(texengine)) {
-      tex <<  "\\fontsize{" << pentype.size() << "}{" << pentype.Lineskip()
-	  << "}\\selectfont\n";
-      wait(tex,"\n*",abort);
-    }
-  }
-    
-  string font=pentype.Font();
-  if(font != lastpen.Font()) {
-    tex <<  font << "\n";
-    wait(tex,"\n*",abort);
-  }
-    
-  lastpen=pentype;
-    
-  bool nullsize=size.empty();
-  if(!texbounds(tex,label,abort,nullsize) && !nullsize)
-    texbounds(tex,size,abort,false);
+  setpen(tex,texengine,pentype);
+  texbounds(width,height,depth,tex,label);
+  
+  if(width == 0.0 && height == 0.0 && depth == 0.0 && !size.empty())
+    texbounds(width,height,depth,tex,size);
+
   enabled=true;
     
   Align=inverse(T)*align;
   double scale0=max(fabs(Align.getx()),fabs(Align.gety()));
   if(scale0) Align *= 0.5/scale0;
   Align -= pair(0.5,0.5);
-  double Depth=(pentype.Baseline() == NOBASEALIGN) ? depth : 0.0;
+  double Depth=(pentype.Baseline() == NOBASEALIGN) ? depth : 
+    -depth*Align.gety();
   texAlign=Align;
-  if(Depth > 0) texAlign += pair(0.0,Depth/(height+Depth));
-  Align.scale(width,height+Depth);
+  const double vertical=height+depth;
+  if(Depth > 0) texAlign += pair(0.0,Depth/vertical);
+  Align.scale(width,vertical);
   Align += pair(0.0,Depth-depth);
   Align=T*Align;
 }
 
 void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
-		       bboxlist&)
+                       bboxlist&)
 {
   string texengine=getSetting<string>("tex");
   if(texengine == "none") {b += position; return;}
@@ -167,7 +124,7 @@ void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
   // alignment point
   pair p=position+Align;
   const double vertical=height+depth;
-  const double fuzz=pentype.size()*0.05+0.3;
+  const double fuzz=pentype.size()*0.1+0.3;
   pair A=p+T*pair(-fuzz,-fuzz);
   pair B=p+T*pair(-fuzz,vertical+fuzz);
   pair C=p+T*pair(width+fuzz,vertical+fuzz);
@@ -178,34 +135,34 @@ void drawLabel::bounds(bbox& b, iopipestream& tex, boxvector& labelbounds,
     box Box=box(A,B,C,D);
     for(size_t i=0; i < n; i++) {
       if(labelbounds[i].intersect(Box)) {
-	switch(pentype.Overwrite()) {
-	case SUPPRESS:
-	  labelwarning("suppressed");
-	case SUPPRESSQUIET:
-	  suppress=true; 
-	  return;
-	case MOVE:
-	  labelwarning("moved");
-	default:
-	  break;
-	}
+        switch(pentype.Overwrite()) {
+          case SUPPRESS:
+            labelwarning("suppressed");
+          case SUPPRESSQUIET:
+            suppress=true; 
+            return;
+          case MOVE:
+            labelwarning("moved");
+          default:
+            break;
+        }
 
-	pair Align=(align == pair(0,0)) ? unit(pair(urand(),urand())) :
-	  unit(align);
-	double s=0.1*pentype.size();
-	double dx=0, dy=0;
-	if(Align.getx() > 0.1) dx=labelbounds[i].xmax()-Box.xmin()+s;
-	if(Align.getx() < -0.1) dx=labelbounds[i].xmin()-Box.xmax()-s;
-	if(Align.gety() > 0.1) dy=labelbounds[i].ymax()-Box.ymin()+s;
-	if(Align.gety() < -0.1) dy=labelbounds[i].ymin()-Box.ymax()-s;
-	pair offset=pair(dx,dy);
-	position += offset;
-	A += offset;
-	B += offset;
-	C += offset;
-	D += offset;
-	Box=box(A,B,C,D);
-	i=0;
+        pair Align=(align == pair(0,0)) ? unit(pair(urand(),urand())) :
+          unit(align);
+        double s=0.1*pentype.size();
+        double dx=0, dy=0;
+        if(Align.getx() > 0.1) dx=labelbounds[i].xmax()-Box.xmin()+s;
+        if(Align.getx() < -0.1) dx=labelbounds[i].xmin()-Box.xmax()-s;
+        if(Align.gety() > 0.1) dy=labelbounds[i].ymax()-Box.ymin()+s;
+        if(Align.gety() < -0.1) dy=labelbounds[i].ymin()-Box.ymax()-s;
+        pair offset=pair(dx,dy);
+        position += offset;
+        A += offset;
+        B += offset;
+        C += offset;
+        D += offset;
+        Box=box(A,B,C,D);
+        i=0;
       }
     }
     labelbounds.resize(n+1);
@@ -239,7 +196,7 @@ bool drawLabel::write(texfile *out, const bbox&)
 drawElement *drawLabel::transformed(const transform& t)
 {
   return new drawLabel(label,size,t*T,t*position,
-		       length(align)*unit(shiftless(t)*align),pentype);
+                       length(align)*unit(shiftless(t)*align),pentype);
 }
 
 void drawLabelPath::bounds(bbox& b, iopipestream& tex, boxvector&, bboxlist&)
@@ -308,7 +265,7 @@ bool drawLabelPath::write(texfile *out, const bbox&)
 drawElement *drawLabelPath::transformed(const transform& t)
 {
   return new drawLabelPath(label,size,transpath(t),justify,shift,
-			   transpen(t));
+                           transpen(t));
 }
 
 } //namespace camp
