@@ -83,7 +83,7 @@ public:
  
   void render(GLUnurbs *nurb, double size2,
               const triple &Min, const triple& Max, double perspective,
-              bool transparent) const;
+              bool lighton, bool transparent) const;
   bool shipout3(const string& prefix, const string& format,
                 double width, double height, double angle, double zoom,
                 const triple& m, const triple& M, const pair& shift, double *t,
@@ -92,7 +92,7 @@ public:
                 bool viewportlighting, bool view);
   
   // PRC output
-  bool shipout3(const string& prefix, vm::array *index, vm::array *center);
+  bool shipout3(const string& prefix);
   
   bool reloadPDF(const string& Viewer, const string& outname) const;
   
@@ -116,7 +116,7 @@ inline picture *transformed(const vm::array& t, picture *p)
 }
 
 void texinit();
-int opentex(const string& texname, const string& prefix);
+int opentex(const string& texname, const string& prefix, bool dvi=false);
 
 const char *texpathmessage();
   
