@@ -15,11 +15,15 @@
 #include "settings.h"
 #include "errormsg.h"
 #include "array.h"
+#include "stack.h"
 
 using std::ofstream;
 using std::setw;
 using vm::array;
 using vm::read;
+using vm::stack;
+using vm::callable;
+using vm::pop;
 
 namespace camp {
 
@@ -43,11 +47,13 @@ psfile::psfile(const string& filename, bool pdfformat)
 {
   if(filename.empty()) out=&cout;
   else out=new ofstream(filename.c_str());
+  out->setf(std::ios::boolalpha);
   if(!out || !*out)
     reportError("Cannot write to "+filename);
 }
 
 static const char *inconsistent="inconsistent colorspaces";
+static const char *rectangular="matrix is not rectangular";
   
 void psfile::writefromRGB(unsigned char r, unsigned char g, unsigned char b, 
                           ColorSpace colorspace, size_t ncomponents) 
@@ -117,6 +123,9 @@ void psfile::close()
   if(out) {
     out->flush();
     if(!filename.empty()) {
+#ifdef __MSDOS__  
+      chmod(filename.c_str(),~settings::mask & 0777);
+#endif	    
       if(!out->good())
         // Don't call reportError since this may be called on handled_error.
         reportFatal("Cannot write to "+filename);
@@ -142,7 +151,7 @@ void psfile::prologue(const bbox& box)
   header();
   BoundingBox(box);
   *out << "%%Creator: " << settings::PROGRAM << " " << settings::VERSION
-       << SVN_REVISION <<  newl;
+       << REVISION <<  newl;
 
   time_t t; time(&t);
   struct tm *tt = localtime(&t);
@@ -218,28 +227,26 @@ void psfile::setpen(pen p)
   else setcolor(p);
   
   // Defer dynamic linewidth until stroke time in case currentmatrix changes.
-  if(p.width() != lastpen.width()) {
+  if(p.width() != lastpen.width())
     *out << p.width() << (pdfformat ? " setlinewidth" : " Setlinewidth") 
          << newl;
-  }
     
-  if(p.cap() != lastpen.cap()) {
+  if(p.cap() != lastpen.cap())
     *out << p.cap() << " setlinecap" << newl;
-  }
     
-  if(p.join() != lastpen.join()) {
+  if(p.join() != lastpen.join())
     *out << p.join() << " setlinejoin" << newl;
-  }
     
-  if(p.miter() != lastpen.miter()) {
+  if(p.miter() != lastpen.miter())
     *out << p.miter() << " setmiterlimit" << newl;
-  }
-    
-  if(p.stroke() != lastpen.stroke() || 
-     p.linetype().offset != lastpen.linetype().offset) {
+
+  const LineType *linetype=p.linetype();
+  const LineType *lastlinetype=lastpen.linetype();
+  
+  if(!(linetype->pattern == lastlinetype->pattern) || 
+     linetype->offset != lastlinetype->offset) {
     out->setf(std::ios::fixed);
-    *out << "[" << p.stroke() << "] " << std::setprecision(6) 
-         << p.linetype().offset << " setdash" << newl;
+    *out << linetype->pattern << " " << linetype->offset << " setdash" << newl;
     out->unsetf(std::ios::fixed);
   }
     
@@ -283,7 +290,7 @@ void psfile::write(path p, bool newPath)
   }
 }
 
-void psfile::latticeshade(const vm::array& a, const bbox& b)
+void psfile::latticeshade(const vm::array& a, const transform& t)
 {
   checkLevel();
   size_t n=a.size();
@@ -300,8 +307,7 @@ void psfile::latticeshade(const vm::array& a, const bbox& b)
   
   *out << "<< /ShadingType 1" << newl
        << "/Matrix ";
-
-  write(matrix(b.Min(),b.Max()));
+  write(t);
   *out << newl;
   *out << "/ColorSpace /Device" << ColorDeviceSuffix[colorspace] << newl
        << "/Function" << newl
@@ -320,33 +326,35 @@ void psfile::latticeshade(const vm::array& a, const bbox& b)
   *out << "/Size [" << m << " " << n << "]" << newl
        << "/DataSource <" << newl;
 
-  beginHex();
   for(size_t i=n; i > 0;) {
     array *ai=read<array *>(a,--i);
     checkArray(ai);
     size_t aisize=ai->size();
-    if(aisize != m) reportError("shading matrix must be rectangular");
+    if(aisize != m) reportError(rectangular);
     for(size_t j=0; j < m; j++) {
       pen *p=read<pen *>(ai,j);
       p->convert();
       if(!p->promote(colorspace))
         reportError(inconsistent);
-      writeHex(p,ncomponents);
+      *out << p->hex() << newl;
     }
   }
-  endHex();
-
-  *out << ">>" << newl
+  
+  *out << ">" << newl
+       << ">>" << newl
        << ">>" << newl
        << "shfill" << newl;
 }
 
 // Axial and radial shading
-void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
+void psfile::gradientshade(bool axial, ColorSpace colorspace,
                            const pen& pena, const pair& a, double ra,
-                           const pen& penb, const pair& b, double rb)
+                           bool extenda, const pen& penb, const pair& b,
+                           double rb, bool extendb)
 {
   checkLevel();
+  endclip(pena);
+
   setopacity(pena);
   checkColorSpace(colorspace);
   
@@ -358,7 +366,7 @@ void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
   write(b);
   if(!axial) write(rb);
   *out << "]" << newl
-       << "/Extend [true true]" << newl
+       << "/Extend [" << extenda << " " << extendb << "]" << newl
        << "/Function" << newl
        << "<< /FunctionType 2" << newl
        << "/Domain [0 1]" << newl
@@ -374,10 +382,13 @@ void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
        << "shfill" << newl;
 }
   
-void psfile::gouraudshade(const array& pens, const array& vertices,
+void psfile::gouraudshade(const pen& pentype,
+                          const array& pens, const array& vertices,
                           const array& edges)
 {
   checkLevel();
+  endclip(pentype);
+
   size_t size=pens.size();
   if(size == 0) return;
   
@@ -403,11 +414,23 @@ void psfile::gouraudshade(const array& pens, const array& vertices,
        << "shfill" << newl;
 }
  
+void psfile::vertexpen(array *pi, int j, ColorSpace colorspace)
+{
+  pen *p=read<pen *>(pi,j);
+  p->convert();
+  if(!p->promote(colorspace))
+    reportError(inconsistent);
+  *out << " ";
+  write(*p);
+}
+
 // Tensor-product patch shading
-void psfile::tensorshade(const array& pens, const array& boundaries,
-                         const array& z)
+void psfile::tensorshade(const pen& pentype, const array& pens,
+                         const array& boundaries, const array& z)
 {
   checkLevel();
+  endclip(pentype);
+  
   size_t size=pens.size();
   if(size == 0) return;
   size_t nz=z.size();
@@ -432,10 +455,10 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
     path g=read<path>(boundaries,i);
     if(!(g.cyclic() && g.size() == 4))
       reportError("specify cyclic path of length 4");
-    for(Int j=0; j < 4; ++j) {
+    for(Int j=4; j > 0; --j) {
       write(g.point(j));
-      write(g.postcontrol(j));
-      write(g.precontrol(j+1));
+      write(g.precontrol(j));
+      write(g.postcontrol(j-1));
     }
     if(nz == 0) { // Coons patch
       static double nineth=1.0/9.0;
@@ -449,21 +472,19 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
       array *zi=read<array *>(z,i);
       if(checkArray(zi) != 4)
         reportError("specify 4 internal control points for each path");
-      for(Int j=0; j < 4; ++j)
-        write(read<pair>(zi,j));
+      write(read<pair>(zi,0));
+      write(read<pair>(zi,3));
+      write(read<pair>(zi,2));
+      write(read<pair>(zi,1));
     }
     
     array *pi=read<array *>(pens,i);
     if(checkArray(pi) != 4)
       reportError("specify 4 pens for each path");
-    for(Int j=0; j < 4; ++j) {
-      pen *p=read<pen *>(pi,j);
-      p->convert();
-      if(!p->promote(colorspace))
-        reportError(inconsistent);
-      *out << " ";
-      write(*p);
-    }
+    vertexpen(pi,0,colorspace);
+    vertexpen(pi,3,colorspace);
+    vertexpen(pi,2,colorspace);
+    vertexpen(pi,1,colorspace);
     *out << newl;
   }
   
@@ -472,15 +493,6 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
        << "shfill" << newl;
 }
  
-inline unsigned char byte(double r) // Map [0,1] to [0,255]
-{
-  if(r < 0.0) r=0.0;
-  else if(r > 1.0) r=1.0;
-  int a=(int)(256.0*r);
-  if(a == 256) a=255;
-  return a;
-}
-
 void psfile::write(pen *p, size_t ncomponents) 
 {
   switch(ncomponents) {
@@ -504,32 +516,6 @@ void psfile::write(pen *p, size_t ncomponents)
   }
 }
 
-void psfile::writeHex(pen *p, size_t ncomponents) 
-{
-  switch(ncomponents) {
-    case 0:
-      break;
-    case 1: 
-      write2(byte(p->gray())); 
-      writenewl();
-      break;
-    case 3:
-      write2(byte(p->red())); 
-      write2(byte(p->green())); 
-      write2(byte(p->blue())); 
-      writenewl();
-      break;
-    case 4:
-      write2(byte(p->cyan())); 
-      write2(byte(p->magenta())); 
-      write2(byte(p->yellow())); 
-      write2(byte(p->black())); 
-      writenewl();
-    default:
-      break;
-  }
-}
-
 string filter() 
 {
   return settings::getSetting<Int>("level") >= 3 ? 
@@ -582,7 +568,10 @@ void psfile::image(const array& a, const array& P, bool antialias)
   double max=min;
   for(size_t i=0; i < asize; i++) {
     array *ai=read<array *>(a,i);
-    for(size_t j=0; j < a0size; j++) {
+    size_t size=ai->size();
+    if(size != a0size)
+        reportError(rectangular);
+    for(size_t j=0; j < size; j++) {
       double val=read<double>(ai,j);
       if(val > max) max=val;
       else if(val < min) min=val;
@@ -628,7 +617,10 @@ void psfile::image(const array& a, bool antialias)
   beginImage(ncomponents*a0size*asize);
   for(size_t i=0; i < asize; i++) {
     array *ai=read<array *>(a,i);
-    for(size_t j=0; j < a0size; j++) {
+    size_t size=ai->size();
+    if(size != a0size)
+        reportError(rectangular);
+    for(size_t j=0; j < size; j++) {
       pen *p=read<pen *>(ai,j);
       p->convert();
       if(!p->promote(colorspace))
@@ -639,6 +631,41 @@ void psfile::image(const array& a, bool antialias)
   endImage(antialias,a0size,asize,ncomponents);
 }
   
+void psfile::image(stack *Stack, callable *f, Int width, Int height,
+                   bool antialias)
+{
+  if(width <= 0 || height <= 0) return;
+  
+  Stack->push(0);
+  Stack->push(0);
+  f->call(Stack);
+  pen p=pop<pen>(Stack);
+
+  setopacity(p);
+  
+  ColorSpace colorspace=p.colorspace();
+  checkColorSpace(colorspace);
+  
+  size_t ncomponents=ColorComponents[colorspace];
+  
+  imageheader(width,height,colorspace);
+    
+  beginImage(ncomponents*width*height);
+  for(Int j=0; j < height; j++) {
+    for(Int i=0; i < width; i++) {
+      Stack->push(j);
+      Stack->push(i);
+      f->call(Stack);
+      pen p=pop<pen>(Stack);
+      p.convert();
+      if(!p.promote(colorspace))
+        reportError(inconsistent);
+      write(&p,ncomponents);
+    }
+  }
+  endImage(antialias,width,height,ncomponents);
+}
+  
 void psfile::outImage(bool antialias, size_t width, size_t height,
                       size_t ncomponents)
 {
