@@ -867,7 +867,7 @@ void addOption(option *o) {
 
 void version()
 {
-  cerr << PROGRAM << " version " << VERSION << SVN_REVISION
+  cerr << PROGRAM << " version " << REVISION
        << " [(C) 2004 Andy Hammerlindl, John C. Bowman, Tom Prince]" 
        << endl;
 }
@@ -1147,6 +1147,8 @@ void initSettings() {
   addOption(new versionOption("version", 0, "Show version"));
 
   addOption(new pairSetting("offset", 'O', "pair", "PostScript offset"));
+  addOption(new pairSetting("aligndir", 0, "pair",
+                             "Directional page alignment (overrides align)"));
   addOption(new alignSetting("align", 'a', "C|B|T|Z",
                              "Center, Bottom, Top, or Zero page alignment",
                              "C"));
@@ -1162,7 +1164,7 @@ void initSettings() {
   addOption(new boolSetting("keepaux", 0,
                             "Keep intermediate LaTeX .aux files"));
   addOption(new engineSetting("tex", 0, "engine",
-                              "latex|pdflatex|xelatex|tex|pdftex|luatex|lualatex|none",
+                              "latex|pdflatex|xelatex|lualatex|tex|pdftex|luatex|context|none",
                               "latex"));
 
   addOption(new boolSetting("twice", 0,
@@ -1329,8 +1331,15 @@ void setInteractive()
      (isatty(STDIN_FILENO) || getSetting<Int>("inpipe") >= 0))
     interact::interactive=true;
   
-  historyname=getSetting<bool>("localhistory") ? 
-    (string(getPath())+dirsep+"."+suffix+"_history") : (initdir+"/history");
+  if(getSetting<bool>("localhistory"))
+    historyname=string(getPath())+dirsep+"."+suffix+"_history";
+  else {
+    if(mkdir(initdir.c_str(),0777) != 0 && errno != EEXIST)
+      cerr << "failed to create directory "+initdir+"." << endl;
+    historyname=initdir+"/history";
+  }
+  if(verbose > 1)
+     cerr << "Using history " << historyname << endl;
 }
 
 bool view()
@@ -1399,9 +1408,10 @@ void initDir() {
   if(mask == 0) mask=0027;
   umask(mask);
 #endif  
-  if(verbose > 1)
-    cerr << "Using configuration directory " << initdir << endl;
-  mkdir(initdir.c_str(),0777);
+  if(access(initdir.c_str(),F_OK) == 0) {
+    if(verbose > 1)
+      cerr << "Using configuration directory " << initdir << endl;
+  }
 }
   
 void setPath() {
@@ -1416,7 +1426,8 @@ void setPath() {
     }
     if(i < asydir.length()) searchPath.push_back(asydir.substr(i));
   }
-  searchPath.push_back(initdir);
+  if(access(initdir.c_str(),F_OK) == 0)
+    searchPath.push_back(initdir);
   string sysdir=getSetting<string>("sysdir");
   if(sysdir != "")
     searchPath.push_back(sysdir);
@@ -1569,9 +1580,8 @@ Int getScroll()
     if(!terminal)
       terminal=getenv("TERM");
     if(terminal) {
-      int error;
-      error=setupterm(terminal,1,&error);
 #ifndef __MSDOS__      
+      int error=setupterm(terminal,1,&error);
       if(error == 0) scroll=lines > 2 ? lines-1 : 1;
       else
 #endif
