@@ -98,6 +98,9 @@ void texfile::prologue()
     }
   }
   
+  if(settings::xe(texengine) && !inlinetex)
+    *out << "\\usepackage{everypage}%" << newl;
+  
   if(settings::latex(texengine)) {
     *out << "\\setlength{\\unitlength}{1pt}" << newl;
     if(!inlinetex) {
@@ -114,7 +117,7 @@ void texfile::prologue()
   } else {
     if(!inlinetex) {
       if(settings::context(texengine)) {
-        *out << "\\setuplayout[width=16383pt,height=16383pt,"
+        *out << "\\setuplayout["
              << "backspace=0pt,topspace=0pt,"
              << "header=0pt,headerdistance=0pt,footer=0pt]" << newl
              << "\\setuppagenumbering[location=]" << endl
@@ -132,6 +135,7 @@ void texfile::prologue()
       }
     }
   }
+  
   beginpage();
 }
     
@@ -311,7 +315,7 @@ void texfile::put(const string& label, const transform& T, const pair& z,
   if(trans)
     *out << "{" << T.getxx() << " " << sign*T.getyx()
          << " " << sign*T.getxy() << " " << T.getyy() << "}";
-  *out << "{" << label << "}" << newl;
+  *out << "{" << label << "}%" << newl;
 }
 
 void texfile::epilogue(bool pipe)
@@ -345,9 +349,9 @@ void svgtexfile::endspecial()
   
 void svgtexfile::begintransform()
 {
-  *out << "<g transform='matrix(1 0 0 1 "
-       << (-Hoffset+1.99*settings::cm)*ps2tex << " " 
-       << (1.9*settings::cm+box.top)*ps2tex 
+  *out << "<g transform='matrix(" << tex2ps << " 0 0 " << tex2ps <<" "
+       << (-Hoffset+1.99*settings::cm) << " " 
+       << (1.9*settings::cm+box.top) 
        << ")'>" << nl;
 }
     
@@ -539,7 +543,9 @@ void svgtexfile::begingradientshade(bool axial, ColorSpace colorspace,
 
 void svgtexfile::gradientshade(bool axial, ColorSpace colorspace,
                                const pen& pena, const pair& a, double ra,
-                               const pen& penb, const pair& b, double rb)
+                               bool, const pen& penb, const pair& b,
+                               double rb, bool)
+
 {
   *out << "' fill='url(#grad" << gradientcount << ")'";
   fillrule(pena);
