@@ -43,7 +43,7 @@ enum ty_kind {
  
 #define PRIMITIVE(name,Name,asyName) ty_##name,
 #define PRIMERROR
-#include <primitives.h>
+#include "primitives.h"
 #undef PRIMERROR
 #undef PRIMITIVE
 
@@ -53,16 +53,18 @@ enum ty_kind {
 // Forward declarations.
 class ty;
 struct signature;
+typedef mem::vector<ty *> ty_vector;
+typedef ty_vector::iterator ty_iterator;
 
 // Checks if two types are equal in the sense of the language.
 // That is primitive types are equal if they are the same kind.
 // Structures are equal if they come from the same struct definition.
 // Arrays are equal if their cell types are equal.
-bool equivalent(ty *t1, ty *t2);
+bool equivalent(const ty *t1, const ty *t2);
 
 // If special is true, this is the same as above.  If special is false, just the
 // signatures are compared.
-bool equivalent(ty *t1, ty *t2, bool special);
+bool equivalent(const ty *t1, const ty *t2, bool special);
 
 class caster {
 public:
@@ -79,9 +81,9 @@ public:
   virtual ~ty();
 
   virtual void print (ostream& out) const;
-  virtual void printVar (ostream& out, symbol *name) const {
+  virtual void printVar (ostream& out, string name) const {
     print(out);
-    out << " " << *name;
+    out << " " << name;
   }
 
 
@@ -96,10 +98,33 @@ public:
     return 0;
   }
 
+  virtual const signature *getSignature() const {
+    return 0;
+  }
+
   virtual bool primitive() {
     return false;
   }
 
+  bool isError() const { return kind == ty_error; }
+  bool isNotError() const { return !isError(); }
+
+  // The following are only used by the overloaded type, but it is so common
+  // to test for an overloaded type then iterate over its types, that this
+  // allows the code:
+  // if (t->isOverloaded()) {
+  //   for (ty_iterator i = t->begin(); i != t->end(); ++i) {
+  //     ...
+  //   }
+  // }
+  // For speed reasons, only begin has an assert to test if t is overloaded.
+  bool isOverloaded() const {
+    return kind == ty_overloaded;
+  }
+  bool isNotOverloaded() const { return !isOverloaded(); }
+  ty_iterator begin();
+  ty_iterator end();
+
   // If a default initializer is not stored in the environment, the abstract
   // syntax asks the type if it has a "default" default initializer, by calling
   // this method.
@@ -125,14 +150,14 @@ public:
   // parameter and returns the necessary result.
   // These should not have public permission, as modifying them would
   // have strange results.
-  virtual trans::varEntry *virtualField(symbol *, signature *) {
+  virtual trans::varEntry *virtualField(symbol, signature *) {
     return 0;
   }
 
   // varGetType for virtual fields.
   // Unless you are using functions for virtual fields, the base implementation
   // should work fine.
-  virtual ty *virtualFieldGetType(symbol *id);
+  virtual ty *virtualFieldGetType(symbol id);
 
 #if 0
   // Returns the type.  In case of functions, return the equivalent type
@@ -146,7 +171,7 @@ public:
   // Returns true if the other type is equivalent to this one.
   // The general function equivalent should be preferably used, as it properly
   // handles overloaded type comparisons.
-  virtual bool equiv(ty *other)
+  virtual bool equiv(const ty *other) const
   {
     return this==other;
   }
@@ -154,7 +179,7 @@ public:
 
   // Returns a number for the type for use in a hash table.  Equivalent types
   // must yield the same number.
-  virtual size_t hash() = 0;
+  virtual size_t hash() const = 0;
 };
 
 class primitiveTy : public ty {
@@ -170,14 +195,15 @@ public:
     return false;
   }
   
-  virtual trans::varEntry *virtualField(symbol *, signature *);
+  ty *virtualFieldGetType(symbol );
+  trans::varEntry *virtualField(symbol, signature *);
 
-  bool equiv(ty *other)
+  bool equiv(const ty *other) const
   {
     return this->kind==other->kind;
   }
 
-  size_t hash() {
+  size_t hash() const {
     return (size_t)kind + 47;
   }
 };
@@ -193,7 +219,7 @@ public:
 
   trans::access *castTo(ty *target, caster &);
 
-  size_t hash() {
+  size_t hash() const {
     return (size_t)kind + 47;
   }
 };
@@ -218,12 +244,12 @@ struct array : public ty {
     return true;
   }
 
-  bool equiv(ty *other) {
+  bool equiv(const ty *other) const {
     return other->kind==ty_array &&
       equivalent(this->celltype,((array *)other)->celltype);
   }
 
-  size_t hash() {
+  size_t hash() const {
     return 1007 * celltype->hash();
   }
 
@@ -249,71 +275,54 @@ struct array : public ty {
   // NOTE: General vectorization of casts would be here.
 
   // Add length and push as virtual fields.
-  ty *virtualFieldGetType(symbol *id);
-  trans::varEntry *virtualField(symbol *id, signature *sig);
+  ty *virtualFieldGetType(symbol id);
+  trans::varEntry *virtualField(symbol id, signature *sig);
 };
 
 /* Base types */
-#define PRIMITIVE(name,Name,asyName) ty *prim##Name();
+#define PRIMITIVE(name,Name,asyName) \
+  ty *prim##Name(); \
+  ty *name##Array(); \
+  ty *name##Array2(); \
+  ty *name##Array3();
 #define PRIMERROR
-#include <primitives.h>
+#include "primitives.h"
 #undef PRIMERROR
 #undef PRIMITIVE
 
 ty *primNull();
-  
-ty *boolArray();
-ty *IntArray();
-ty *realArray();
-ty *pairArray();
-ty *tripleArray();
-ty *stringArray();
-ty *transformArray();
-ty *pathArray();
-ty *penArray();
-ty *guideArray();
 
-ty *boolArray2();
-ty *IntArray2();
-ty *realArray2();
-ty *pairArray2();
-ty *tripleArray2();
-ty *stringArray2();
-ty *pathArray2();
-ty *penArray2();
 
-ty *boolArray3();
-ty *IntArray3();
-ty *realArray3();
-ty *pairArray3();
-ty *tripleArray3();
-ty *stringArray3();
-  
 struct formal {
   ty *t;
-  symbol *name;
-  absyntax::varinit *defval;
+  symbol name;
+  bool defval;
   bool Explicit;
   
   formal(ty *t,
-         symbol *name=0,
+         symbol name=symbol::nullsym,
          bool optional=false,
          bool Explicit=false)
     : t(t), name(name),
-      defval(optional ? absyntax::Default : 0), Explicit(Explicit) {}
+      defval(optional), Explicit(Explicit) {}
 
+  // string->symbol translation is costly if done too many times.  This
+  // constructor has been disabled to make this cost more visible to the
+  // programmer.
+#if 0
   formal(ty *t,
          const char *name,
          bool optional=false,
          bool Explicit=false)
     : t(t), name(symbol::trans(name)),
       defval(optional ? absyntax::Default : 0), Explicit(Explicit) {}
+#endif
 
   friend ostream& operator<< (ostream& out, const formal& f);
 };
 
-bool equivalent(formal& f1, formal& f2);
-bool argumentEquivalent(formal &f1, formal& f2);
+bool equivalent(const formal& f1, const formal& f2);
+bool argumentEquivalent(const formal &f1, const formal& f2);
 
 typedef mem::vector<formal> formal_vector;
 
@@ -322,6 +331,10 @@ typedef mem::vector<formal> formal_vector;
 struct signature : public gc {
   formal_vector formals;
 
+  // The number of keyword-only formals.  These formals always come after the
+  // regular formals.
+  size_t numKeywordOnly;
+
   // Formal for the rest parameter.  If there is no rest parameter, then the
   // type is null.
   formal rest;
@@ -329,15 +342,16 @@ struct signature : public gc {
   bool isOpen;
 
   signature()
-    : rest(0), isOpen(false)
+    : numKeywordOnly(0), rest(0), isOpen(false)
   {}
 
   static const struct OPEN_t {} OPEN;
 
-  explicit signature(OPEN_t) : rest(0), isOpen(true) {}
+  explicit signature(OPEN_t) : numKeywordOnly(0), rest(0), isOpen(true) {}
 
   signature(signature &sig)
-    : formals(sig.formals), rest(sig.rest), isOpen(sig.isOpen)
+    : formals(sig.formals), numKeywordOnly(sig.numKeywordOnly),
+      rest(sig.rest), isOpen(sig.isOpen)
   {}
 
   virtual ~signature() {}
@@ -346,18 +360,27 @@ struct signature : public gc {
     formals.push_back(f);
   }
 
+  void addKeywordOnly(formal f) {
+    add(f);
+    ++numKeywordOnly;
+  }
+
   void addRest(formal f) {
     rest=f;
   }
 
-  bool hasRest() {
+  bool hasRest() const {
     return rest.t;
   }
-  size_t getNumFormals() {
+  size_t getNumFormals() const {
     return rest.t ? formals.size() + 1 : formals.size();
   }
 
-  formal& getFormal(size_t n) {
+  formal& getFormal(size_t n) { 
+    assert(n < formals.size());
+    return formals[n];
+  }
+  const formal& getFormal(size_t n) const {
     assert(n < formals.size());
     return formals[n];
   }
@@ -365,21 +388,30 @@ struct signature : public gc {
   formal& getRest() {
     return rest;
   }
+  const formal& getRest() const {
+    return rest;
+  }
+
+  bool formalIsKeywordOnly(size_t n) const
+  {
+    assert(n < formals.size());
+    return n >= formals.size() - numKeywordOnly;
+  }
 
   friend ostream& operator<< (ostream& out, const signature& s);
 
-  friend bool equivalent(signature *s1, signature *s2);
+  friend bool equivalent(const signature *s1, const signature *s2);
 
   // Check if a signature of argument types (as opposed to formal parameters)
   // are equivalent.  Here, the arguments, if named, must have the same names,
   // and (for simplicity) no overloaded arguments are allowed.
-  friend bool argumentEquivalent(signature *s1, signature *s2);
+  friend bool argumentEquivalent(const signature *s1, const signature *s2);
 #if 0
   friend bool castable(signature *target, signature *source);
   friend Int numFormalsMatch(signature *s1, signature *s2);
 #endif
 
-  size_t hash();
+  size_t hash() const;
 };
 
 struct function : public ty {
@@ -428,7 +460,7 @@ struct function : public ty {
     return true;
   }
 
-  bool equiv(ty *other)
+  bool equiv(const ty *other) const
   {
     if (other->kind==ty_function) {
       function *that=(function *)other;
@@ -438,14 +470,14 @@ struct function : public ty {
     else return false;
   }
 
-  size_t hash() {
+  size_t hash() const {
     return sig.hash()*0x1231+result->hash();
   }
 
   void print(ostream& out) const
   { out << *result << sig; }
 
-  void printVar (ostream& out, symbol *name) const {
+  void printVar (ostream& out, string name) const {
     result->printVar(out,name);
     out << sig;
   }
@@ -458,6 +490,10 @@ struct function : public ty {
     return &sig;
   }
 
+  const signature *getSignature() const {
+    return &sig;
+  }
+
 #if 0
   ty *stripDefaults();
 #endif
@@ -466,26 +502,29 @@ struct function : public ty {
   trans::access *initializer();
 };
 
-typedef mem::vector<ty *> ty_vector;
-
 // This is used in getType expressions when an overloaded variable is accessed.
 class overloaded : public ty {
 public:
   ty_vector sub;
+
+  // Warning: The venv endScope routine relies heavily on the current
+  // implementation of overloaded.
 public:
   overloaded()
     : ty(ty_overloaded) {}
+  overloaded(ty *t)
+    : ty(ty_overloaded) { add(t); }
   virtual ~overloaded() {}
 
-  bool equiv(ty *other)
+  bool equiv(const ty *other) const
   {
-    for(ty_vector::iterator i=sub.begin();i!=sub.end();++i)
+    for(ty_vector::const_iterator i=sub.begin();i!=sub.end();++i)
       if (equivalent(*i,other))
         return true;
     return false;
   }
 
-  size_t hash() {
+  size_t hash() const {
     // Overloaded types should not be hashed.
     assert(False);
     return 0;
@@ -525,9 +564,19 @@ public:
   // True if one of the subtypes is castable.
   bool castable(ty *target, caster &c);
 
+  size_t size() const { return sub.size(); }
+
   // Use default printing for now.
 };
 
+inline ty_iterator ty::begin() {
+  assert(this->isOverloaded());
+  return ((overloaded *)this)->sub.begin();
+}
+inline ty_iterator ty::end() {
+  return ((overloaded *)this)->sub.end();
+}
+
 // This is used to encapsulate iteration over the subtypes of an overloaded
 // type.  The base method need only be implemented to handle non-overloaded
 // types.
